@@ -12,7 +12,8 @@ import logging
 
 from . import log
 
-logger = logging.getLogger('app')
+logger = logging.getLogger("app")
+
 
 class CustomEncoder(json.JSONEncoder):
     def default(self, obj):
@@ -21,6 +22,7 @@ class CustomEncoder(json.JSONEncoder):
         else:
             return super().default(obj)
 
+
 #######################################################
 # Config object to handle a number of helper functions
 #######################################################
@@ -31,6 +33,7 @@ class Config(object):
     lookup etc.
 
     """
+
     def __init__(self, params, *args, **kwargs):
         """
         Initialize the config (context) object.
@@ -41,37 +44,30 @@ class Config(object):
                          settings is the path to the settings file.
         """
         super().__init__(*args, **kwargs)
-        self.debug = params.get('debug', False)
-        self.settings = params.get('settings', None)
-        self.readonly = params.get('readonly', False)
-        self.show_metadata = params.get('show_metadata', True)
+        self.debug = params.get("debug", False)
+        self.settings = params.get("settings", None)
+        self.readonly = params.get("readonly", False)
+        self.show_metadata = params.get("show_metadata", True)
 
         if self.settings is not None:
             self.settings = os.path.expanduser(self.settings)
 
         self.now = datetime.now()
-        self.conf = {
-            'credentials': {},
-            'libraries': []
-        }
+        self.conf = {"credentials": {}, "libraries": []}
         self.stats = {
-            'platform': {
-                'node': platform.node(),
-                'os': platform.system(),
-                'release': platform.release(),
-                'processor': platform.processor(),
-                'python': platform.python_version(),
-                'distribution': distro.linux_distribution()
+            "platform": {
+                "node": platform.node(),
+                "os": platform.system(),
+                "release": platform.release(),
+                "processor": platform.processor(),
+                "python": platform.python_version(),
+                "distribution": distro.linux_distribution(),
             },
-            'user': getpass.getuser(),
+            "user": getpass.getuser(),
         }
 
         self.state = self.get_default_metadata()
-        self.state.update({
-            'outputs': [],
-            'notes': [],
-            'message': ""
-        })
+        self.state.update({"outputs": [], "notes": [], "message": ""})
 
     def get_default_metadata(self):
         """
@@ -80,18 +76,18 @@ class Config(object):
         metadata = {
             "schema": "standalone:command",
             "version": "1.0",
-            'timestamp': datetime.now().replace(microsecond=0).isoformat(),
-            'command': {
+            "timestamp": datetime.now().replace(microsecond=0).isoformat(),
+            "command": {
                 "name": "Unknown",
-                'status': 'initial',
+                "status": "initial",
                 "description": "",
-                "host":  self.stats['platform']['node'],
+                "host": self.stats["platform"]["node"],
                 "stats": self.stats,
-                "pid":  os.getpid(),
-                'start_time': self.now,
-                'end_time': self.now,
-                "cmdline":  list(sys.argv),
-            }
+                "pid": os.getpid(),
+                "start_time": self.now,
+                "end_time": self.now,
+                "cmdline": list(sys.argv),
+            },
         }
 
         return metadata
@@ -107,8 +103,7 @@ class Config(object):
         if settings is None:
             settings = self.settings
 
-        if ((settings is None) or
-            (not os.path.exists(settings))):
+        if (settings is None) or (not os.path.exists(settings)):
             # Quietly return. Do nothing..
             raise Exception("Settings path invalid or undefined: {}".format(settings))
 
@@ -122,13 +117,13 @@ class Config(object):
             return
 
         # Materialize the paths...
-        self.enrich_root = loaded_conf.get('enrich_root')
+        self.enrich_root = loaded_conf.get("enrich_root")
         self.enrich_root = os.path.expanduser(self.enrich_root)
         self.enrich_root = os.path.abspath(self.enrich_root)
 
-        libraries = loaded_conf.get('libraries', [])
+        libraries = loaded_conf.get("libraries", [])
         libraries = [self.get_file(l) for l in libraries]
-        self.conf['libraries'] = libraries
+        self.conf["libraries"] = libraries
 
         # Insert the rest
         defaults = {
@@ -140,13 +135,19 @@ class Config(object):
             "outputdir": "%(rundir)s/output",
             "log": "%(rundir)s/log.json",
             "metadata": "%(rundir)s/metadata.json",
-            'args': {}
+            "args": {},
         }
 
-        for name in ['enrich_data', 'enrich_customers',
-                     'runid', 'command_root',
-                     'rundir', 'outputdir', 'log',
-                     'metadata']:
+        for name in [
+            "enrich_data",
+            "enrich_customers",
+            "runid",
+            "command_root",
+            "rundir",
+            "outputdir",
+            "log",
+            "metadata",
+        ]:
             default = defaults[name]
             try:
                 final = loaded_conf.get(name, default)
@@ -156,12 +157,12 @@ class Config(object):
                 raise Exception(error)
 
             setattr(self, name, final)
-            self.state['command'][name] = final
+            self.state["command"][name] = final
 
-        for name in ['args']:
+        for name in ["args"]:
             default = defaults[name]
             try:
-                final = loaded_conf.get(name,default)
+                final = loaded_conf.get(name, default)
             except:
                 error = "Could not resolve: {} ({})".format(name, final)
                 raise Exception(error)
@@ -169,9 +170,9 @@ class Config(object):
             setattr(self, name, final)
 
         # Set environment variables.....
-        os.environ['ENRICH_ROOT'] = self.enrich_root
-        os.environ['ENRICH_CUSTOMERS'] = self.enrich_customers
-        os.environ['ENRICH_DATA'] = self.enrich_data
+        os.environ["ENRICH_ROOT"] = self.enrich_root
+        os.environ["ENRICH_CUSTOMERS"] = self.enrich_customers
+        os.environ["ENRICH_DATA"] = self.enrich_data
 
     def get_file(self, path):
         """
@@ -187,28 +188,27 @@ class Config(object):
         This is resolved into `/home/ubuntu/enrich/data/acme/Projects/commands`
 
         """
-        path = path.replace("%(",'%%(') # This will make the strftime handling safe
+        path = path.replace("%(", "%%(")  # This will make the strftime handling safe
         path = self.now.strftime(path)
         path = path % {
-            'enrich_root': self.enrich_root,
-            'enrich_data': getattr(self, 'enrich_data', ''),
-            'enrich_customers': getattr(self, 'enrich_customers', ''),
-            'user': self.stats['user'],
-            'username': self.stats['user'],
-            'hostname': self.stats['platform']['node'],
-            'node': self.stats['platform']['node'],
-            'runid': getattr(self, 'runid', ''),
-            'rundir': getattr(self, 'rundir', ''),
-            'outputdir': getattr(self, 'outputdir', ''),
-
+            "enrich_root": self.enrich_root,
+            "enrich_data": getattr(self, "enrich_data", ""),
+            "enrich_customers": getattr(self, "enrich_customers", ""),
+            "user": self.stats["user"],
+            "username": self.stats["user"],
+            "hostname": self.stats["platform"]["node"],
+            "node": self.stats["platform"]["node"],
+            "runid": getattr(self, "runid", ""),
+            "rundir": getattr(self, "rundir", ""),
+            "outputdir": getattr(self, "outputdir", ""),
         }
         return path
 
-    def get_relative_path(self, path,what='enrich_root'):
+    def get_relative_path(self, path, what="enrich_root"):
 
-        if what == 'enrich_root':
+        if what == "enrich_root":
             return os.path.relpath(path, self.enrich_root)
-        elif what == 'enrich_data':
+        elif what == "enrich_data":
             return os.path.relpath(path, self.enrich_data)
         else:
             raise Exception("relative path should specify the root")
@@ -223,23 +223,27 @@ class Config(object):
               to the file, and a description for the output
 
         """
-        required = ['name', 'description', 'path']
+        required = ["name", "description", "path"]
         missing = [n for n in required if n not in params]
         if len(missing) > 0:
             raise Exception("Missing parameters for the output: {}".format(missing))
 
-        if os.path.isabs(params['path']):
-            raise Exception("Path specified for output {} should be relative".format(params['name']))
+        if os.path.isabs(params["path"]):
+            raise Exception(
+                "Path specified for output {} should be relative".format(params["name"])
+            )
 
-        for o in self.state['outputs']:
-            if o['name'] == params['name']:
-                raise Exception("An output already exists with the name: {}".format(params['name']))
+        for o in self.state["outputs"]:
+            if o["name"] == params["name"]:
+                raise Exception(
+                    "An output already exists with the name: {}".format(params["name"])
+                )
 
-        params['fullpath'] = self.get_file("%(outputdir)s/{}".format(params['path']))
-        params['relpath'] = self.get_relative_path(params['fullpath'])
+        params["fullpath"] = self.get_file("%(outputdir)s/{}".format(params["path"]))
+        params["relpath"] = self.get_relative_path(params["fullpath"])
 
         # Now append
-        self.state['outputs'].append(params)
+        self.state["outputs"].append(params)
 
     def get_output_handle(self, name, *args):
         """
@@ -250,19 +254,18 @@ class Config(object):
               This output was previously added using `add_output` call.
 
         """
-        for o in self.state['outputs']:
-            if o['name'] == name:
+        for o in self.state["outputs"]:
+            if o["name"] == name:
                 try:
-                    os.makedirs(os.path.dirname(o['fullpath']))
+                    os.makedirs(os.path.dirname(o["fullpath"]))
                 except:
                     pass
-                return open(o['fullpath'], *args)
+                return open(o["fullpath"], *args)
 
         raise Exception("Could not find output {}".format(name))
 
     def get_library_paths(self):
-        return self.conf['libraries']
-
+        return self.conf["libraries"]
 
     def get_args(self, cmd):
         """
@@ -272,8 +275,8 @@ class Config(object):
            cmd (str): Name of the command
 
         """
-        logger = logging.getLogger('app')
-        if not hasattr(self, 'args'):
+        logger = logging.getLogger("app")
+        if not hasattr(self, "args"):
             if self.debug:
                 logger.error("args not present")
             return {}
@@ -287,32 +290,34 @@ class Config(object):
 
     # Update the state
     def set_status(self, value):
-        self.state['command']['status'] = value
+        self.state["command"]["status"] = value
 
     def set_message(self, value):
-        self.state['message'] = value
+        self.state["message"] = value
 
     def dump_state(self):
 
         # Update file statistics
-        for o in self.state['outputs']:
-            path = o['fullpath']
+        for o in self.state["outputs"]:
+            path = o["fullpath"]
             st = os.stat(path)
-            o['size'] = st.st_size
+            o["size"] = st.st_size
 
-        metadata = OrderedDict([
-            ('schema', self.state['schema']),
-            ('version', self.state['version']),
-            ('timestamp', self.state['timestamp']),
-            ('command', self.state['command']),
-            ("message", self.state['message']),
-            ("outputs", self.state['outputs']),
-            ("notes", self.state['notes']),
-            ("log", os.path.relpath(self.log, self.rundir)),
-        ])
+        metadata = OrderedDict(
+            [
+                ("schema", self.state["schema"]),
+                ("version", self.state["version"]),
+                ("timestamp", self.state["timestamp"]),
+                ("command", self.state["command"]),
+                ("message", self.state["message"]),
+                ("outputs", self.state["outputs"]),
+                ("notes", self.state["notes"]),
+                ("log", os.path.relpath(self.log, self.rundir)),
+            ]
+        )
 
         if not self.readonly:
-            with open(self.metadata, 'w') as fd:
+            with open(self.metadata, "w") as fd:
                 json.dump(metadata, fd, indent=4, cls=CustomEncoder)
         elif self.show_metadata:
             print(json.dumps(metadata, indent=4, cls=CustomEncoder), file=sys.stderr)
@@ -326,8 +331,8 @@ class Config(object):
         # Override with parameters from settings
         if not self.readonly:
             args = self.get_args(cmd)
-            self.readonly = args.get('readonly', self.readonly)
-            self.show_metadata = args.get('show_metadata', self.show_metadata)
+            self.readonly = args.get("readonly", self.readonly)
+            self.show_metadata = args.get("show_metadata", self.show_metadata)
 
         # Setup logging
         if self.readonly:
@@ -335,7 +340,7 @@ class Config(object):
         else:
             log.logging_start(self.log)
 
-        logger = logging.getLogger('app')
+        logger = logging.getLogger("app")
         if self.debug:
             logger.debug("Starting run")
 
@@ -344,14 +349,12 @@ class Config(object):
 
     def end_run(self):
 
-        self.state['command']['end_time'] = datetime.now()
+        self.state["command"]["end_time"] = datetime.now()
         self.dump_state()
 
         # Dump status
-        logger = logging.getLogger('app')
+        logger = logging.getLogger("app")
 
         if self.debug:
             logger.debug("Ending run")
         log.logging_end()
-
-
