@@ -22,12 +22,14 @@ from functools import partial
 logger = logging.getLogger("app")
 
 from enrichsdk.utils import get_lineage_of_query
+from enrichsdk.lib.exceptions import NoDataFound
+
 
 def note(df, title):
     msg = title + "\n"
     msg += "--------" + "\n"
     msg += "Timestamp: " + str(datetime.now()) + "\n"
-    msg += "\nShape: "+ str(df.shape) + "\n"
+    msg += "\nShape: " + str(df.shape) + "\n"
     msg += "\nColumns: " + ", ".join(df.columns) + "\n"
     if len(df) > 0:
         msg += "\nSample:" + "\n"
@@ -37,21 +39,24 @@ def note(df, title):
     msg += "------" + "\n"
     return msg
 
-class CustomEncoder(json.JSONEncoder):
 
+class CustomEncoder(json.JSONEncoder):
     def default(self, obj):
         try:
             return super().default(obj)
         except:
             return str(obj)
 
+
 def get_yesterday():
     yesterday = date.today() + timedelta(days=-1)
     return yesterday.isoformat()
 
+
 def get_today():
     return date.today().isoformat()
 
+
 class MetricsBase(Compute):
     """
     Compute metrics as input for the anomaly/other computation
@@ -62,22 +67,16 @@ class MetricsBase(Compute):
         * Highlevel specification of dimensions and metrics
 
     """
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.name = "MetricsBase"
         self.description = "Compute metrics against datasources"
         self.testdata = {
-            'data_root': os.path.join(os.environ['ENRICH_TEST'],
-                                      self.name),
-            'statedir': os.path.join(os.environ['ENRICH_TEST'],
-                                     self.name, 'state'),
-            'conf': {
-                'args': {
-
-                }
-            },
-            'data': {
-            }
+            "data_root": os.path.join(os.environ["ENRICH_TEST"], self.name),
+            "statedir": os.path.join(os.environ["ENRICH_TEST"], self.name, "state"),
+            "conf": {"args": {}},
+            "data": {},
         }
 
     @classmethod
@@ -89,9 +88,9 @@ class MetricsBase(Compute):
         pretty print the URL
         """
         username = engine.url.username
-        host          = engine.url.host
-        database   = engine.url.database
-        drivername= engine.url.get_driver_name()
+        host = engine.url.host
+        database = engine.url.database
+        drivername = engine.url.get_driver_name()
 
         return f"{drivername}:///{host}/{database}/"
 
@@ -99,7 +98,7 @@ class MetricsBase(Compute):
         """
         Return database URI for a source
         """
-        return source['uri']
+        return source["uri"]
 
     def get_handlers(self, profile):
         """
@@ -113,23 +112,27 @@ class MetricsBase(Compute):
         Read the profile json
         """
 
-        if ((not hasattr(self, 'profiledir')) and (not hasattr(self, 'profilefile'))):
-            raise Exception("'profiledir' transform attribute should be defined to use default get_profile method")
+        if (not hasattr(self, "profiledir")) and (not hasattr(self, "profilefile")):
+            raise Exception(
+                "'profiledir' transform attribute should be defined to use default get_profile method"
+            )
 
         paths = []
-        if hasattr(self, 'profilefile'):
+        if hasattr(self, "profilefile"):
             paths.append(self.profilefile)
 
-        if hasattr(self, 'profiledir'):
-            paths.extend([
-                self.profiledir + "/profile.json",
-                self.profiledir + "/profile.yaml",
-                self.profiledir + "/profilespec.json",
-                self.profiledir + "/profilespec.yaml",
-            ])
+        if hasattr(self, "profiledir"):
+            paths.extend(
+                [
+                    self.profiledir + "/profile.json",
+                    self.profiledir + "/profile.yaml",
+                    self.profiledir + "/profilespec.json",
+                    self.profiledir + "/profilespec.yaml",
+                ]
+            )
 
         profile = None
-        for p  in paths:
+        for p in paths:
             if not os.path.exists(p):
                 continue
             if p.endswith(".json"):
@@ -140,30 +143,29 @@ class MetricsBase(Compute):
         if profile is None:
             raise Exception("Profile could not be found")
 
-        sources = profile['sources']
-        specs = profile['specs']
-        logger.debug(f"Found {len(sources)} sources {len(specs)} specs",
-                     extra={
-                         'transform': self.name,
-                         'data': json.dumps(profile, indent=4)
-                     })
+        sources = profile["sources"]
+        specs = profile["specs"]
+        logger.debug(
+            f"Found {len(sources)} sources {len(specs)} specs",
+            extra={"transform": self.name, "data": json.dumps(profile, indent=4)},
+        )
 
         return profile
 
     def get_specs(self, profile):
-        if ((not isinstance(profile, dict)) or ('specs' not in profile)):
+        if (not isinstance(profile, dict)) or ("specs" not in profile):
             raise Exception("Specs not defined in profile")
-        return profile['specs']
+        return profile["specs"]
 
     def get_sources(self, profile):
-        if ((not isinstance(profile, dict)) or ('sources' not in profile)):
+        if (not isinstance(profile, dict)) or ("sources" not in profile):
             raise Exception("Sources not defined in profile")
-        return profile['sources']
+        return profile["sources"]
 
     def get_db_query(self, source):
 
         # Generate/extract the query...
-        query = source['query']
+        query = source["query"]
         if callable(query):
             query = quer(source)
         return query
@@ -174,8 +176,8 @@ class MetricsBase(Compute):
             name = source
             description = ""
         else:
-            name = source['name']
-            description = source.get('description','')
+            name = source["name"]
+            description = source.get("description", "")
 
         if self.state.has_frame(name):
             return
@@ -188,13 +190,13 @@ class MetricsBase(Compute):
                 params.extend(lineage)
 
         detail = {
-            'name': name,
-            'df': df,
-            'frametype': 'pandas',
-            'description': description,
-            'params': params,
-            'transform': self.name,
-            'history': []
+            "name": name,
+            "df": df,
+            "frametype": "pandas",
+            "description": description,
+            "params": params,
+            "transform": self.name,
+            "history": [],
         }
 
         self.state.update_frame(name, detail)
@@ -221,9 +223,9 @@ class MetricsBase(Compute):
                 {
                     "type": "database",
                     "nature": "input",
-                    "objects": [self.get_printable_db_uri(uri)]
+                    "objects": [self.get_printable_db_uri(uri)],
                 },
-            ]
+            ],
         }
 
         self.update_frame(source, df, lineage)
@@ -236,25 +238,18 @@ class MetricsBase(Compute):
         """
 
         if not isinstance(profile, dict) or len(profile) == 0:
-            logger.warning("Empty profile",
-                           extra={
-                               'transform': self.name
-                           })
+            logger.warning("Empty profile", extra={"transform": self.name})
             return {}
 
         # Get various kinds of handlers..
         handlers = self.get_handlers(profile)
         if not isinstance(handlers, dict) or len(handlers) == 0:
-            logger.warning("No handlers specified",
-                           extra={
-                               'transform': self.name
-                           })
+            logger.warning("No handlers specified", extra={"transform": self.name})
             handlers = {}
 
-
         required_sources = []
         for s in specs:
-            required_sources.extend(s['sources'])
+            required_sources.extend(s["sources"])
         required_sources = list(set(required_sources))
 
         # Now no about constructucting the datasets
@@ -263,33 +258,34 @@ class MetricsBase(Compute):
         sources = self.get_sources(profile)
         for source in sources:
 
-            nature = source.get('nature', 'db')
-            name = source['name']
+            nature = source.get("nature", "db")
+            name = source["name"]
 
             if name not in required_sources:
                 continue
 
-            pipeline = source.get('pipeline', None)
-            generate = source.get('generate', None)
+            pipeline = source.get("pipeline", None)
+            generate = source.get("generate", None)
 
             # Only db is used for now...
             try:
-                if nature == 'db':
+                if nature == "db":
                     result = self.read_db_source(source)
-                elif ((generate is not None) and
-                      (generate in handlers) and
-                      (callable(handlers[generate]))):
+                elif (
+                    (generate is not None)
+                    and (generate in handlers)
+                    and (callable(handlers[generate]))
+                ):
                     result = handlers[generate](source)
-                elif ((generate is not None) and
-                      (hasattr(self, generate))):
+                elif (generate is not None) and (hasattr(self, generate)):
                     result = getattr(self, generate)(source)
                 else:
                     raise Exception(f"Invalid specification: {name}")
             except:
-                logger.exception(f"[{name}] generation failed",
-                                 extra={
-                                     'transform': self.name
-                                 })
+                logger.exception(
+                    f"[{name}] generation failed", extra={"transform": self.name}
+                )
+                continue
 
             # Clean the read the dataset...
             try:
@@ -305,51 +301,54 @@ class MetricsBase(Compute):
                         elif callable(processor):
                             result = processor(result, source)
                         else:
-                            raise Exception("Only method names/callables are supported are supported")
+                            raise Exception(
+                                "Only method names/callables are supported are supported"
+                            )
             except:
-                logger.exception(f"[{name}] post-processing failed",
-                                 extra={
-                                     'transform': self.name
-                                 })
+                logger.exception(
+                    f"[{name}] post-processing failed", extra={"transform": self.name}
+                )
+                continue
+
             # We could return multiple values or a single value
             if isinstance(result, dict):
                 datasets.update(result)
             else:
                 datasets[name] = df
 
-
         return datasets
 
     def get_spec_sources(self, spec, datasets):
 
-        name = spec['name']
+        name = spec["name"]
 
-        if (('sources' not in spec) and ('source' not in spec)):
+        if ("sources" not in spec) and ("source" not in spec):
             raise Exception(f"[{name}] Invalid specification. Missing dataset")
 
-        sources = spec.get('sources', spec.get('source'))
+        sources = spec.get("sources", spec.get("source"))
         if isinstance(sources, str):
             sources = [sources]
 
+        policy = spec.get("missing", "fail")
         for s in sources:
             if s not in datasets:
-                raise Exception(f"[{name}] Missing source: {s}")
+                if policy == "fail":
+                    raise Exception(f"[{name}] Missing source: {s}")
 
-        return {s : datasets[s] for s in sources}
+        return {s: datasets[s] for s in sources if s in datasets}
 
     def process_spec(self, datasets, profile, spec):
 
-        if (('name' not in spec) or
-            ('description' not in spec)):
+        if ("name" not in spec) or ("description" not in spec):
             raise Exception("Invalid spec: name/description missing")
 
-        name = spec['name']
+        name = spec["name"]
         handlers = self.get_handlers(profile)
 
-        if ((not isinstance(spec, dict)) or (len(spec) == 0)):
+        if (not isinstance(spec, dict)) or (len(spec) == 0):
             raise Exception("Spec should be a dict")
 
-        if 'generate' not in spec:
+        if "generate" not in spec:
             # It is database operational specification..
             data = self.process_spec_default(datasets, profile, spec)
         else:
@@ -358,11 +357,9 @@ class MetricsBase(Compute):
 
         # The spec processor can return multiple dataframes
         if isinstance(data, pd.DataFrame):
-            data = {
-                name: data
-            }
+            data = {name: data}
 
-        pipeline = spec.get('pipeline', None)
+        pipeline = spec.get("pipeline", None)
         if pipeline is not None:
             for processor in pipeline:
                 if isinstance(processor, str):
@@ -371,23 +368,25 @@ class MetricsBase(Compute):
                     elif hasattr(self, processor):
                         data = getattr(self, processor)(data, spec)
                     else:
-                            raise Exception(f"Missing cleaner: {processor}")
+                        raise Exception(f"Missing cleaner: {processor}")
                 elif callable(processor):
                     data = processor(data, spec)
                 else:
-                    raise Exception("Only method names/callables are supported are supported")
+                    raise Exception(
+                        "Only method names/callables are supported are supported"
+                    )
 
         return
 
     def process_spec_custom(self, datasets, profile, spec):
 
-        name = spec['name']
+        name = spec["name"]
         handlers = self.get_handlers(profile)
 
         # Custom...
-        generate = spec['generate']
+        generate = spec["generate"]
         callback = None
-        if ((generate in handlers) and (not callable(handlers[generate]))):
+        if (generate in handlers) and (not callable(handlers[generate])):
             callback = handlers[generate]
         elif hasattr(self, generate):
             callback = getattr(self, generate)
@@ -405,12 +404,10 @@ class MetricsBase(Compute):
         Handle one specification at a time..
         """
 
-        if (('dimensions' not in spec) or
-            (not isinstance(spec['dimensions'], dict))):
+        if ("dimensions" not in spec) or (not isinstance(spec["dimensions"], dict)):
             raise Exception("Dimensions in spec should be a dict")
 
-        if (('metrics' not in spec) or
-            (not isinstance(spec['metrics'], dict))):
+        if ("metrics" not in spec) or (not isinstance(spec["metrics"], dict)):
             raise Exception("Metrics in spec should be a dict")
 
         # Get hold of the data first...
@@ -422,8 +419,8 @@ class MetricsBase(Compute):
         datasetdf = list(sources.values())[0]
 
         # now go through each of the dimensions
-        dimensions = spec['dimensions']
-        metrics = spec['metrics']
+        dimensions = spec["dimensions"]
+        metrics = spec["metrics"]
 
         _dfs = []
         for name, cols in dimensions.items():
@@ -456,6 +453,7 @@ class MetricsBase(Compute):
                     what = "_".join(what)
                     what = what.rstrip("_").lstrip("_")
                 return what
+
             df.columns = df.columns.map(clean_colname)
 
             df.insert(0, "dimensions", name)
@@ -466,30 +464,29 @@ class MetricsBase(Compute):
         df = pd.concat(_dfs)
         del _dfs
 
-        return {
-            spec['name']: df
-        }
+        return {spec["name"]: df}
 
     def get_dataset_generic(self, source):
         """
         Use the dataset object to read the dataset
         """
 
-        if ((not hasattr(self, 'read_data')) or
-            (not hasattr(self, 'get_dataset'))):
-            raise Exception(" get_dataset_raw expects read_data and get_dataset methods")
+        if (not hasattr(self, "read_data")) or (not hasattr(self, "get_dataset")):
+            raise Exception(
+                " get_dataset_raw expects read_data and get_dataset methods"
+            )
 
         args = self.args
-        start_date = self.args['start_date']
-        end_date = self.args['end_date']
+        start_date = self.args["start_date"]
+        end_date = self.args["end_date"]
 
-        name = source['name']
-        dataset = source['dataset']
-        params = source.get('params', {})
-        setdate = source.get('date', False)
+        name = source["name"]
+        dataset = source["dataset"]
+        params = source.get("params", {})
+        setdate = source.get("date", False)
 
-        cache = args.get('cache', False)
-        cachename= f"{dataset}-{start_date}-{end_date}"
+        cache = args.get("cache", False)
+        cachename = f"{dataset}-{start_date}-{end_date}"
         cachefile = f"cache/{self.name}-cache-" + cachename + ".csv"
 
         if cache:
@@ -498,28 +495,24 @@ class MetricsBase(Compute):
             except:
                 pass
             if os.path.exists(cachefile):
-                logger.debug("Read cached {}".format(name),
-                             extra={
-                                 'transform': self.name
-                             })
+                logger.debug(
+                    "Read cached {}".format(name), extra={"transform": self.name}
+                )
 
                 df = pd.read_csv(cachefile, **params)
-                return {
-                    name: df
-                }
+                return {name: df}
 
         datasetobj = self.get_dataset(dataset)
-        df, metadata = datasetobj.read_data(start_date,
-                                            end_date,
-                                            filename=source['filename'],
-                                            readfunc=self.read_data,
-                                            params=params
+
+        df, metadata = datasetobj.read_data(
+            start_date,
+            end_date,
+            filename=source["filename"],
+            readfunc=self.read_data,
+            params=params,
         )
 
-        logger.debug("Read {}".format(name),
-                     extra={
-                         'transform': self.name
-                     })
+        logger.debug("Read {}".format(name), extra={"transform": self.name})
 
         # Cache it for future use...
         if cache:
@@ -527,7 +520,7 @@ class MetricsBase(Compute):
 
         # Insert lineage if possible
         lineage = None
-        if (('files' in metadata) and (len(metadata['files']) > 0)):
+        if ("files" in metadata) and (len(metadata["files"]) > 0):
             lineage = {
                 "type": "lineage",
                 "transform": self.name,
@@ -535,25 +528,22 @@ class MetricsBase(Compute):
                     {
                         "type": "file",
                         "nature": "input",
-                        "objects": [metadata['files'][-1]]
+                        "objects": [metadata["files"][-1]],
                     },
-                ]
+                ],
             }
 
         self.update_frame(source, df, lineage)
 
-        return {
-            name: df
-        }
+        return {name: df}
 
     def process(self, state):
         """
         Run the computation and update the state
         """
-        logger.debug("Start execution",
-                     extra=self.config.get_extra({
-                         'transform': self.name
-                     }))
+        logger.debug(
+            "Start execution", extra=self.config.get_extra({"transform": self.name})
+        )
 
         # Will be used in other places..
         self.state = state
@@ -568,16 +558,15 @@ class MetricsBase(Compute):
 
         # Now go through each spec and get the output..
         for spec in specs:
-            enable = spec.get('enable', True)
+            enable = spec.get("enable", True)
             if not enable:
                 continue
             self.process_spec(datasets, profile, spec)
 
         # Done
-        logger.debug("Complete execution",
-                     extra=self.config.get_extra({
-                         'transform': self.name
-                     }))
+        logger.debug(
+            "Complete execution", extra=self.config.get_extra({"transform": self.name})
+        )
 
         ###########################################
         # => Return
