@@ -240,7 +240,7 @@ class S3Mixin(CloudMixin):
         if 'region' in cred:
             if client_kwargs is None:
                 client_kwargs = {}
-            client_kwargs['region_name'] = cred.get('region'),
+            client_kwargs['region_name'] = cred.get('region')
 
         return s3fs.S3FileSystem(
             anon=False,
