@@ -477,7 +477,7 @@ class Dataset(DataSource):
         ):
             raise Exception("Matching requirement {} missing/invalid".format(compare))
 
-    def generate(self, start, end=None):
+    def generate(self, start, end=None, full=False, name="default", resolve=None):
         """
         Given a start and an end, generate the datetime
         objects corresponding to each run
@@ -485,6 +485,9 @@ class Dataset(DataSource):
         Args:
            start (datetime): Start of the time range
            end (datetime): End of range. If None, default = start
+           full (bool): Whether full path is required or only the suffix. Default is false
+           name (str): Name of the path specification. Optional is full path is required
+           resolve (dict): Additional path resolution parameters
 
         Returns:
            list (dict): List of dictionaries (name, timestamp)
@@ -510,6 +513,15 @@ class Dataset(DataSource):
         # Generate all possible names
         names = func(start, end)
 
+        # Now insert the full path...
+        try:
+            if full:
+                path_root = self.get_path_by_name(name, resolve=resolve)
+                for n in names:
+                    n['path'] = os.path.join(path_root, n['name'])
+        except:
+            logger.exception("Unable to resolve names")
+
         # Check if the name is in the names..
         return names
 
