@@ -6,6 +6,7 @@ import sys
 import pip
 import base64
 import subprocess
+import textwrap
 import json
 import imp
 import yaml
@@ -70,8 +71,7 @@ def _version():
 
 
 @click.command("start", hidden=True)
-@click.option("--commands/--no-commands", default=False, help="Show commands")
-def _start(commands):
+def _start():
     """
     Instructions to get started
     """
@@ -83,11 +83,11 @@ def _start(commands):
     if "ENRICH_ROOT" in os.environ:
         enrich_root = os.environ["ENRICH_ROOT"]
     else:
-        enrich_root = "$ENRICH_ROOT"
+        enrich_root = os.getcwd() + "/enrich"
 
     if "VIRTUAL_ENV" in os.environ:
         venv_root = os.environ["VIRTUAL_ENV"]
-        venv_name = os.path.basename(venv_root)
+        venv_name = os.path.abspath(os.path.basename(venv_root))
         install_settings = [
             'echo "source {}/bin/activate" > $ENRICH_ROOT/env.sh'.format(venv_root),
             'echo "#workon {}" >> $ENRICH_ROOT/env.sh'.format(venv_name),
@@ -107,33 +107,34 @@ def _start(commands):
             "description": "Understand Enrich and SDK",
         },
         {
-            "description": "set ENRICH_ROOT and populate",
+            "description": "set ENRICH_ROOT and populate. This will create the directory structure required to make enrich work. You will see several directories that are required at various points in the lifecycle of the project",
             "cmd": [
-                "export ENRICH_ROOT={}/work/enrich".format(home),
-                "mkdir -p $ENRICH_ROOT",
+                f"export ENRICH_ROOT={enrich_root}",
                 "enrichpkg env populate",
             ],
         },
         {
-            "description": "Check and update siteconf and versionmap",
+            "description": "Check and update siteconf and versionmap. Siteconf stores site-specific configuration required by pipelines including credentials, security policy etc.. Versionmap (which you dont have to worry about right now) allows the pipelines to track the version information of all the modules deployed",
             "cmd": [
                 "enrichpkg env check",
                 "cat $ENRICH_ROOT/etc/siteconf.json",
             ],
         },
         {
-            "description": "[OPTIONAL] For complex deployments, create an environment/context file",
+            "description": "[OPTIONAL] For complex deployments, create an environment/context file. Talk to Scribble before you use this.",
             "cmd": "enrichpkg env sample-context > context.yaml",
         },
         {
-            "description": "[OPTIONAl] Create a simple settings file",
+            "description": "[OPTIONAl] Create a simple settings file. This is useful in everyday execution. You can simply source this script and then run the enrichpkg",
             "cmd": [
                 "# file to be sourced before you start working. use the appropriate",
                 "# environment activation mechanism. Check the script generated by",
-                "# populate command",
-                "$ cat $ENRICH_ROOT/env.sh"
+                "# populate",
                 "",
-                "If the file doesnt exist or doesnt look right, use the following"
+                "cat $ENRICH_ROOT/env.sh",
+                'echo "source $ENRICH_ROOT/env.sh" >> $HOME/.bashrc',
+                "",
+                "# If the file doesnt exist or doesnt look right, use the following"
             ]
             + install_settings
             + [
@@ -143,7 +144,7 @@ def _start(commands):
             ],
         },
         {
-            "description": "Change dir to $ENRICH_CUSTOMERS",
+            "description": "Customer directory has actual code repositories. Your code will be stored here. Change dir to $ENRICH_CUSTOMERS. You will find that it is empty but dont panic",
             "cmd": "cd $ENRICH_ROOT/customers",
         },
         {
@@ -151,123 +152,119 @@ def _start(commands):
                 "#To handle python paths etc. avoid spaces and hyphen in the names",
                 "git clone https://github.com/pingali/enrich-acme.git acme",
             ],
-            "description": "GIT checkout a code repository",
+            "description": "GIT checkout an example code repository. This time it is from a company called acme. You will have a similar repository and checkout. The process is similar whether it is on the local machine or on the server",
         },
         {
             "cmd": "cd $ENRICH_ROOT/customers/acme",
             "description": "Change to checked out repository",
         },
         {
-            "cmd": ["enrichpkg bootstrap -c repo -p ."],
-            "description": "Bootstrap the repo if not already done",
+            "cmd": [
+                "# if acme is already populated",
+                "enrichpkg bootstrap -c repo -p ."
+            ],
+            "description": "[OPTIONAL] Bootstrap the repo, if not already done. In case of Acme you dont have tEnrich requires a particular structure to the repository. You will notice __init__.py and enrich.json. These are files that tell the Enrich system what is present where.",
         },
         {
             "cmd": [
                 "# Typically usecases have first letter in capital",
                 "enrichpkg bootstrap -c usecase -p Marketing",
             ],
-            "description": "Create usecase, say Marketing",
+            "description": "[OPTIONAL] The code is organized in terms of projects or usecase groups. Create one usecase, say Marketing. In case of acme, you dont have to do this. Just look around."
         },
-        {"cmd": ["cd Marketing"], "description": "Change to usecase"},
         {
-            "description": "Bootstrap a transform. Transform is a python module. It could be a single file, directory or a full package",
+            "cmd": [
+                "cd Marketing"
+            ],
+            "description": "Change to usecase"
+        },
+        {
+            "description": "Bootstrap a simple transform. Transform is a python module. It could be a single file, directory or a full package.",
             "cmd": [
                 "# Will create an example hello world script",
-                "enrichpkg bootstrap -c transform-helloworld -p transforms/helloworld.py",
-                "",
+                "enrichpkg bootstrap transform-helloworld -p transforms/helloworld.py",
+            ]
+        },
+        {
+            "description": "[OPTIONAL] You could do more complex transforms. You can bootstrap a python module directory or a full package. You can come back to this later on",
+            "cmd": [
                 "# Will create a more simple python script that is more complex than helloworld",
-                "enrichpkg bootstrap -c transform-simple -p transforms/simplefile.py",
+                "enrichpkg bootstrap transform-simple -p transforms/simplefile.py",
                 "",
                 "# Will create a python module (a full directory)",
-                "enrichpkg bootstrap -c transform-simple -p transforms/simpledir",
+                "enrichpkg bootstrap transform-simple -p transforms/simpledir",
                 "",
                 "# Will create a python package. Dont use this unless you are sure",
-                "enrichpkg bootstrap -c transform-package -p transforms/simplepackage",
+                "enrichpkg bootstrap transform-package -p transforms/simplepackage",
             ],
         },
         {
             "cmd": [
                 "# Add any requirements and install them",
-                "vi ../requirements.txt",
+                "cat ../requirements.txt",
                 "pip install -r ../requirements.txt",
             ],
-            "description": "Install repo-specific requirements",
+            "description": "[OPTIONAL] Each repository has additional requirements going beyond what enrich requires, e.g., a module to connect to snowfake. Install these repo-specific requirements to enable the transform to run. You can come back to this later on",
+        },
+        {
+            "cmd": [
+                "vi transforms/helloworld.py",
+            ],
+            "description": "Eyeball the helloworld transform",
         },
         {
             "cmd": [
                 "enrichpkg test transform transforms/helloworld.py",
-                "# To capture all debug logs",
-                "enrichpkg test transform --capture transforms/helloworld.py",
             ],
-            "description": "Test the transform",
+            "description": "You have to locally test the transform to make sure everything is working as planned including the parameters"
         },
         {
             "cmd": [
-                "enrichpkg bootstrap -c pipeline-conf -p pipelines/conf/helloworld.py"
+                "enrichpkg bootstrap pipeline-conf -p pipelines/conf/helloworld.py"
             ],
-            "description": "Bootstrap a pipeline",
+            "description": "Transforms are like plugins. They have to be incorporated into a pipeline to ensure that it runs on server. Bootstrap a pipeline. Note the path to the pipelines. They have a particular structure as described in the SDK and at particular location. The Enrich execution engine searches these paths",
         },
         {
             "cmd": ["enrichpkg test conf --capture pipelines/conf/helloworld.py"],
-            "description": "Check pipeline",
+            "description": "Sanity check the pipeline to ensure that everything is as per plan. Note that in some cases where credentials are being accessed etc. we will have additional setup"
         },
         {
             "cmd": [
                 "# prefect is the default workflow engine",
-                "enrichpkg bootstrap -c prefectjob -p workflows/prefect/daily.py",
+                "enrichpkg bootstrap prefectjob -p workflows/prefect/daily.py",
             ],
-            "description": "Bootstrap a Prefect workflow",
+            "description": "A workflow allows us to combine multiple pipelines and multiple runs of each into a single activity. We use prefect for the same. We could use something else. We dont need prefect cloud though. We will run the workflow from our scheduler"
+        },
+        {
+            "cmd": [
+                "cat workflows/prefect/daily.py",
+                "python3 workflows/prefect/daily.py"
+            ],
+            "description": "Check the content of the workflow. Once ready you can include this workflow in the nextdoor scheduler"
         },
-        {"cmd": ["workflows/prefect/daily.py"], "description": "Check workflow"},
     ]
 
-    if not commands:
-        print("Note: Use --commands to see how to implement these steps\n")
-
     for i, s in enumerate(steps):
-        print("[{:2}] {}".format(i + 1, s["description"]))
-        if not commands:
-            continue
+        print("[{:2}] {}".format(i + 1, "\n".join(textwrap.wrap(s["description"]))))
 
         print("")
         if "cmd" in s:
             if isinstance(s["cmd"], str):
-                print("    $ {}".format(s["cmd"]))
+                cmds = [s["cmd"]]
             else:
-                for c in s["cmd"]:
+                cmds  = s["cmd"]
+            for c in cmds:
+                if not c.startswith("#") and len(c) > 0:
                     print("    $ {}".format(c))
+                else:
+                    print("    {}".format(c))
         print("")
 
 
 @click.command("bootstrap", hidden=True)
+@click.argument("component")
 @click.option(
-    "--component",
-    "-c",
-    required=True,
-    help="What should be bootstrapped",
-    type=click.Choice(
-        [
-            "repo",
-            "usecase",
-            "transform-package",
-            "transform-simple",
-            "transform-helloworld",
-            "transform-query",
-            "asset",
-            "dashboard",
-            "task-lib",
-            "task-conf",
-            "pipeline-conf",
-            "sparkjob",
-            #'airflow',
-            "prefectjob",
-            "rscript",
-            "pyscript",
-        ]
-    ),
-)
-@click.option(
-    "--path", "-p", required=True, help="Directory where the files should be stored"
+    "--path", "-p", required=False, default=None, help="Directory where the files should be stored"
 )
 @click.option(
     "--template",
@@ -276,14 +273,13 @@ def _start(commands):
     default=None,
     help="Template to override the defaults",
 )
-@click.option("--context", default=None, help="Environment file")
+@click.option("--context", required=False, default=None, help="Environment file")
 def bootstrap_(component, path, template, context):
     """
     Bootstrap a fresh module
-    """
 
-    """
-    Modules currently supported include transform, model, and skin.
+    Modules currently supported include transform, pipeline and asset. Use
+    help to list possible components  (enrichpkg bootstrap help)
 
     For now only the transforms and models are testable locally
     without the need for a test server.
@@ -292,8 +288,39 @@ def bootstrap_(component, path, template, context):
     development of all modules.
 
     A custom template can be provided as well.
+
     """
 
+    components = [
+        ["repo" , "A new git repository"],
+        ["usecase", "A collection of transforms, pipelines, assets and workflows"],
+        ["transform-package", "A complex transform that is structured as a full python package"],
+        ["transform-simple", "A transform structured as a python module"],
+        ["transform-helloworld", "A simple hello world transform for study"],
+        ["transform-query", "A  transform that uses a pre-existing database querying module"],
+        ["transform-iris", "A  transform that uses a pre-existing database querying module"],
+        ["asset", "A reusable library"],
+        ["dashboard", "A django app that fits the Enrich Dashboard framework"],
+        ["pipeline-conf", "A pipeline configuration"],
+        ["prefectjob", "Prefect workflow template"],
+        ["rscript", "R-Script template"],
+        ["pyscript", "A python script template"]
+    ]
+
+    if component not in [c[0] for c in components]:
+        print("The following components are supported:\n")
+        for c in components:
+            print(f"   {c[0]:20}: {c[1]}")
+        print("\nUse the component name like this")
+        print("    enrichpkg bootstrap repo -p <path>")
+        print("")
+        print("You can define your own templates as well. Please use the -t option\n")
+        return
+
+    if path is None:
+        print("Path should specified using -p option")
+        return
+
     # use the context file to set the environment
     try:
         context = lib.Context(context)
